# Author : Shubham Sharma
# Contact: shubhamjsharma10@gmail.com

from email.message import Message
from smtplib import SMTP, SMTPException


class EmailService:
    """Email service

    This class serves the purpose of sending the email messages to the clients. 
    Other modules can use this class for sending the email messsages.

    Examples
    --------
    >>> service = EmailService('host', '587', 'John', 'Secret')
    >>> service.sendmail(message)
    """

    def __init__(self,
                 host: str, port: int,
                 user: str, password: str) -> None:
        """Initialize a new instance.

        Parameters
        ----------
        host: str
            The name of remote host

        port: int
            The port to which to connect.

        user: str
            The email address from which to send email messages.

        passwd: str
            The password used for authenticating the email address 
            from which to send the email messages.
        """

        self._host = host
        self._port = port
        self._user = user
        self._password = password


    def sendmail(self, msg: Message) -> None:
        """Sends an email message using a SMTP connection.

        Parameters
        ----------
        msg: Message
            The Message object.

        Returns
        -------
        None
        
        Raises
        ------
        SMTPException
        """

        with SMTP(self._host, self._port) as smtp:
            smtp.ehlo()
            smtp.starttls()
            if self._password:
                smtp.login(self._user, self._password)

            send_errs = smtp.send_message(msg)
            # send_errs is a dictionary, with one entry for each
            # recipient that was refused.  Each entry contains a tuple of the SMTP
            # error code and the accompanying error message sent by the server.



__version__ = '1.0.5'
__author__ = 'Shubham Sharma'
__email__ = 'shubhamjsharma10@gmail.com'
__description__ = 'A simple and convinient way to send emails in python'

__license__ = "Apache License, Version 2.0"
__copyright__ = "2020 %s" % __author__


# Author : Shubham Sharma
# Contact: shubhamjsharma10@gmail.com

"""Messaging utilities

Provides basic messaging utility functions.
"""

import mimetypes
from pathlib import Path
from typing import List, Optional, Union

from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart
from email.message import EmailMessage, Message


def build_message(fr_addr: str,
                  to_addrs: List[str],
                  cc_addrs: List[str],
                  subject: str,
                  content: str,
                  htmlcontent: Optional[str] = None,
                  attachments: Optional[List[str]] = None) -> Message:
    """Creates a new email message.

    Parameters
    ----------
    fr_addr: str
        The sender's email address.

    to_addrs: list of str
        The primary recipient's email addresses.

    cc_addrs: list of str
        The secondary recipient's email addresses.
        If you don't want to specify the secondary recipients just pass in the empty list.

    subject: str
        The subject of message.

    content: str
        The body of email message in plain text format.

    htmlcontent: str, optional
        The body of email message in html format.

    attachments: list of str, optional
        The list of absolute filepaths associated with the files that are present 
        on the filesystem which you wish to attach to the email message.

    Returns
    -------
    Message
    """

    msg = EmailMessage()
    msg['Subject'] = subject
    msg['From'] = fr_addr
    msg['To'] = ', '.join(to_addrs)

    if cc_addrs:
        msg['CC'] = ', '.join(cc_addrs)

    if content is not None:
        msg.set_content(content)
    
    if htmlcontent is not None:
        msg.add_alternative(htmlcontent, subtype='html')

    if attachments is not None:
        for filepath in attachments:
            # Guess the content type.
            ctype, encoding = mimetypes.guess_type(filepath)

            if ctype is None:
                # unknown file type
                # skip the attachment
                continue

            # for example,
            # if ctype is image/png
            # then maintype will be image
            # and subtype will be png
            maintype, subtype = ctype.split('/', maxsplit=1)

            with open(filepath, 'rb') as file:
                msg.add_attachment(file.read(),
                                   maintype=maintype,
                                   subtype=subtype,
                                   filename=Path(filepath).name)

    # Save email message?
    # For now its not required.
    # In general its good practice to save the messages

    return msg


"""A setuptools based setup module.
"""

# Always prefer setuptools over distutils
import setuptools
import pathlib
import shutil

here = pathlib.Path(__file__).parent.resolve()

# Get the long description from the README file
long_description = (here / 'README.md').read_text(encoding='utf-8')

# https://packaging.python.org/en/latest/single_source_version.html
about = {}
exec((here / 'pymailer/__about__.py').read_text(encoding='utf-8'), globals(), about)


dist_path = here / 'dist'
if dist_path.exists():
    shutil.rmtree(dist_path)


setuptools.setup(
    name='py3mailer',
    version=about['__version__'],
    description=about['__description__'],
    long_description=long_description,  # Optional
    long_description_content_type='text/markdown',
    url='https://github.com/complexsum/pymailer',
    author='Shubham Sharma',
    author_email='shubhamjsharma10@gmail.com',
    license='Apache 2.0',

    # Classifiers help users find your project by categorizing it.
    # For a list of valid classifiers, see https://pypi.org/classifiers/
    classifiers=[  # Optional
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 3 - Alpha',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Topic :: Utilities',

        # Pick your license as you wish
        'License :: OSI Approved :: Apache Software License',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate you support Python 3. These classifiers are *not*
        # checked by 'pip install'. See instead 'python_requires' below.
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3 :: Only',

        'Operating System :: OS Independent',
        'Typing :: Typed',
    ],


    keywords='email, python email, python smtp, python send emails, smtp',  # Optional

    # When your source code is in a subdirectory under the project root, e.g.
    # `src/`, it is necessary to specify the `package_dir` argument.
    # package_dir={'': '.'},  # Optional

    # You can just specify package directories manually here if your project is
    # simple. Or you can use find_packages().
    #
    # Alternatively, if you just want to distribute a single Python file, use
    # the `py_modules` argument instead as follows, which will expect a file
    # called `my_module.py` to exist:
    #
    #   py_modules=["my_module"],
    #
    packages=setuptools.find_packages(include=['pymailer', 'pymailer.*'], exclude=['doc']),  # Required

    # Specify which Python versions you support. In contrast to the
    # 'Programming Language' classifiers above, 'pip pip install -i https://test.pypi.org/simple/ py3mailer==1.0.4install' will check this
    # and refuse to install the project if the version does not match. See
    # https://packaging.python.org/guides/distributing-packages-using-setuptools/#python-requires
    python_requires='>=3.6, <4',

    # This field lists other packages that your project depends on to run.
    # Any package you put here will be installed by pip when your project is
    # installed, so they must be valid existing projects.
    #
    # For an analysis of "install_requires" vs pip's requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    # install_requires=['peppercorn'],  # Optional

    project_urls={  # Optional
        'Bug Reports': 'https://github.com/complexsum/pymailer/issues',
        'Source': 'https://github.com/complexsum/pymailer',
    },
)